@@ -1,5 +1,5 @@
 -- Serialize.hs: OpenPGP (RFC4880) serialization (using cereal)
--- Copyright © 2012  Clint Adams
+-- Copyright © 2012-2013  Clint Adams
 -- This software is released under the terms of the ISC license.
 -- (See the LICENSE file).
 
@@ -8,10 +8,10 @@ module Codec.Encryption.OpenPGP.Serialize (
 ) where
 
 import Control.Applicative ((<$>),(<*>))
-import Control.Monad (replicateM, mplus)
-import qualified Crypto.Cipher.RSA as R
-import qualified Crypto.Cipher.DSA as D
-import Data.Bits ((.&.), (.|.), shiftL, shiftR)
+import Control.Monad (guard, mplus, replicateM)
+import qualified Crypto.PubKey.RSA as R
+import qualified Crypto.PubKey.DSA as D
+import Data.Bits ((.&.), (.|.), shiftL, shiftR, testBit)
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC8
@@ -471,7 +471,8 @@ putS2K (IteratedSalted ha salt count) = do
 
 getPacketTypeAndPayload :: Get (Word8, ByteString)
 getPacketTypeAndPayload = do
-    tag <- getWord8 -- FIXME: bit 7 must be 1, check?
+    tag <- getWord8
+    guard (testBit tag 7)
     case tag .&. 0x40 of
         0x00 -> do
                    let t = shiftR (tag .&. 0x3c) 2
@@ -538,7 +539,7 @@ getPkt = do
             | t == 5 = do
                           bs <- getBytes len
                           let ps = flip runGet bs $ do pkp <- getPKPayload
-                                                       ska <- getSKAddendum (pubKeyAlgo pkp)
+                                                       ska <- getSKAddendum pkp
                                                        return $ SecretKeyPkt pkp ska
                           case ps of
                               Left err -> error err
@@ -549,7 +550,7 @@ getPkt = do
             | t == 7 = do
                           bs <- getBytes len
                           let ps = flip runGet bs $ do pkp <- getPKPayload
-                                                       ska <- getSKAddendum (pubKeyAlgo pkp)
+                                                       ska <- getSKAddendum pkp
                                                        return $ SecretSubkeyPkt pkp ska
                           case ps of
                               Left err -> error err
@@ -765,77 +766,62 @@ getPubkey t = fail ("Unsupported pubkey type " ++ show t)
 putPubkey :: PKey -> Put
 putPubkey p = mapM_ put (pubkeyToMPIs p)
 
-getSecretKey :: PubKeyAlgorithm -> Get SKey
-getSecretKey RSA = do MPI d <- get
-                      MPI p <- get
-                      MPI q <- get
-                      MPI _ <- get
-                      let n = p * q
-                      let dP = 0
-                      let dQ = 0
-                      let qinv = 0
-                      return $ RSAPrivateKey (R.PrivateKey 0 n d p q dP dQ qinv)
-getSecretKey RSAEncryptOnly = getSecretKey RSA
-getSecretKey RSASignOnly = getSecretKey RSA
-getSecretKey DSA = do MPI x <- get
-                      return $ DSAPrivateKey (D.PrivateKey (0,0,0) x)
-getSecretKey ElgamalEncryptOnly = getSecretKey Elgamal
-getSecretKey Elgamal = do MPI x <- get
-                          return $ ElGamalPrivateKey [x]
-
--- indefiniteMPIs :: ByteString -> [MPI]
--- indefiniteMPIs bs = do
---     case runGet (many getMPI) bs of
---         Left e -> error e
---         Right mpis -> mpis
+getSecretKey :: PKPayload -> Get SKey
+getSecretKey pkp
+    | _pkalgo pkp `elem` [RSA, RSAEncryptOnly, RSASignOnly] =
+        do MPI d <- get
+           MPI p <- get
+           MPI q <- get
+           MPI _ <- get -- u
+           let n = p * q
+               dP = 0
+               dQ = 0
+               qinv = 0
+               pub = (\(RSAPubKey x) -> x) (_pubkey pkp)
+           return $ RSAPrivateKey (R.PrivateKey pub d p q dP dQ qinv)
+    | _pkalgo pkp == DSA = do MPI x <- get
+                              return $ DSAPrivateKey (D.PrivateKey (0,0,0) x)
+    | _pkalgo pkp `elem` [ElgamalEncryptOnly,Elgamal] =
+        do MPI x <- get
+           return $ ElGamalPrivateKey [x]
 
 putMPI :: MPI -> Put
 putMPI (MPI i) = do let bs = integerToBEBS i
                     putWord16be . countBits $ bs
                     putByteString bs
 
--- getPackets :: Get (Block Pkt)
--- getPackets = Block `fmap` many getPkt
-
--- putPackets :: Block Pkt -> Put
--- putPackets = mapM_ putPkt . unBlock
-
 getPKPayload :: Get PKPayload
 getPKPayload = do
     version <- getWord8
     ctime <- getWord32be
     if version `elem` [2,3] then
-        do v3exp <-  getWord16be
+        do v3e <-  getWord16be
            pka <- get
            pk <- getPubkey pka
-           return $ DeprecatedPubV3 ctime v3exp pka pk
+           return $! PKPayload DeprecatedV3 ctime v3e pka pk
     else
         do pka <- get
            pk <- getPubkey pka
-           return $ PubV4 ctime pka pk
+           return $! PKPayload V4 ctime 0 pka pk
 
 putPKPayload :: PKPayload -> Put
-putPKPayload (DeprecatedPubV3 ctime v3exp pka pk) = do
+putPKPayload (PKPayload DeprecatedV3 ctime v3e pka pk) = do
     putWord8 3
     putWord32be ctime
-    putWord16be v3exp
+    putWord16be v3e
     put pka
     putPubkey pk
-putPKPayload (PubV4 ctime pka pk) = do
+putPKPayload (PKPayload V4 ctime _ pka pk) = do
     putWord8 4
     putWord32be ctime
     put pka
     putPubkey pk
 
-pubKeyAlgo :: PKPayload -> PubKeyAlgorithm
-pubKeyAlgo (DeprecatedPubV3 _ _ pka _) = pka
-pubKeyAlgo (PubV4 _ pka _) = pka
-
-getSKAddendum :: PubKeyAlgorithm -> Get SKAddendum
-getSKAddendum pka = do
+getSKAddendum :: PKPayload -> Get SKAddendum
+getSKAddendum pkp = do
     s2kusage <- getWord8
     case s2kusage of
-        0 -> do sk <- getSecretKey pka
+        0 -> do sk <- getSecretKey pkp
                 checksum <- getWord16be
                 return $ SUUnencrypted sk checksum
         255 -> do symenc <- getWord8
