@@ -1,9 +1,9 @@
 -- Types.hs: OpenPGP (RFC4880) data types
--- Copyright © 2012  Clint Adams
+-- Copyright © 2012-2013  Clint Adams
 -- This software is released under the terms of the ISC license.
 -- (See the LICENSE file).
 
-{-# LANGUAGE ExistentialQuantification, TypeFamilies #-}
+{-# LANGUAGE DeriveDataTypeable, ExistentialQuantification, TemplateHaskell, TypeFamilies #-}
 
 module Codec.Encryption.OpenPGP.Types (
    SigSubPacket(..)
@@ -19,6 +19,7 @@ module Codec.Encryption.OpenPGP.Types (
  , SigType(..)
  , ImageHeader(..)
  , ImageFormat(..)
+ , KeyVersion(..)
  , PKPayload(..)
  , SKAddendum(..)
  , DataType(..)
@@ -61,16 +62,19 @@ module Codec.Encryption.OpenPGP.Types (
  , Verification(..)
 ) where
 
-import qualified Crypto.Cipher.RSA as RSA
-import qualified Crypto.Cipher.DSA as DSA
+import qualified Crypto.PubKey.RSA as RSA
+import qualified Crypto.PubKey.DSA as DSA
 
 import Control.Arrow ((***))
+import Control.Lens (makeLenses)
 import Data.ByteString (ByteString)
-import Data.Char (toLower, toUpper)
 import qualified Data.ByteString as B
-import Data.List.Split (splitEvery)
-import Data.Map (Map)
+import Data.Char (toLower, toUpper)
+import Data.Data (Data)
+import Data.IxSet (IxSet)
+import Data.List.Split (chunksOf)
 import Data.Set (Set)
+import Data.Typeable (Typeable)
 import Data.Word (Word8, Word16, Word32)
 import Numeric (readHex, showHex)
 
@@ -108,7 +112,7 @@ data SymmetricAlgorithm = Plaintext
                         | AES256
                         | Twofish
                         | OtherSA Word8
-     deriving (Show)
+     deriving (Data, Show, Typeable)
 
 instance Eq SymmetricAlgorithm where
     (==) a b = fromFVal a == fromFVal b
@@ -144,7 +148,7 @@ instance FutureVal SymmetricAlgorithm where
 
 data NotationFlag = HumanReadable
                   | OtherNF Int
-     deriving (Show)
+     deriving (Data, Show, Typeable)
 
 instance Eq NotationFlag where
     (==) a b = fromFFlag a == fromFFlag b
@@ -162,7 +166,7 @@ instance FutureFlag NotationFlag where
 data SigSubPacket = SigSubPacket {
     sspCriticality :: Bool
   , sspPayload :: SigSubPacketPayload
-  } deriving Eq
+  } deriving (Data, Eq, Typeable)
 
 instance Show SigSubPacket where
     show x = (if sspCriticality x then "*" else "") ++ (show . sspPayload) x
@@ -192,7 +196,7 @@ data SigSubPacketPayload = SigCreationTime TimeStamp
                   | EmbeddedSignature SignaturePayload
                   | UserDefinedSigSub Word8 ByteString
                   | OtherSigSub Word8 ByteString
-    deriving (Show, Eq) -- FIXME
+    deriving (Data, Eq, Show, Typeable) -- FIXME
 
 data HashAlgorithm = DeprecatedMD5
                    | SHA1
@@ -202,7 +206,7 @@ data HashAlgorithm = DeprecatedMD5
                    | SHA512
                    | SHA224
                    | OtherHA Word8
-    deriving (Show)
+    deriving (Data, Show, Typeable)
 
 instance Eq HashAlgorithm where
     (==) a b = fromFVal a == fromFVal b
@@ -233,7 +237,7 @@ data CompressionAlgorithm = Uncompressed
                           | ZLIB
                           | BZip2
                           | OtherCA Word8
-    deriving (Show)
+    deriving (Show, Data, Typeable)
 
 instance Eq CompressionAlgorithm where
     (==) a b = fromFVal a == fromFVal b
@@ -267,7 +271,7 @@ data PubKeyAlgorithm = RSA
                      | Elgamal
                      | DH
                      | OtherPKA Word8
-    deriving (Show)
+    deriving (Show, Data, Typeable)
 
 instance Eq PubKeyAlgorithm where
     (==) a b = fromFVal a == fromFVal b
@@ -303,7 +307,7 @@ class (Eq a, Ord a) => FutureFlag a where
 
 data KSPFlag = NoModify
              | KSPOther Int
-    deriving (Show)
+    deriving (Data, Show, Typeable)
 
 instance Eq KSPFlag where
     (==) a b = fromFFlag a == fromFFlag b
@@ -326,7 +330,7 @@ data KeyFlag = GroupKey
              | SignDataKey
              | CertifyKeysKey
              | KFOther Int
-    deriving (Show)
+    deriving (Data, Show, Typeable)
 
 instance Eq KeyFlag where
     (==) a b = fromFFlag a == fromFFlag b
@@ -355,7 +359,7 @@ instance FutureFlag KeyFlag where
 
 data RevocationClass = SensitiveRK
                      | RClOther Int
-    deriving (Show)
+    deriving (Data, Show, Typeable)
 
 instance Eq RevocationClass where
     (==) a b = fromFFlag a == fromFFlag b
@@ -376,7 +380,7 @@ data RevocationCode = NoReason
                     | KeyRetiredAndNoLongerUsed
                     | UserIdInfoNoLongerValid
                     | RCoOther Word8
-    deriving (Show)
+    deriving (Show, Data, Typeable)
 
 instance Eq RevocationCode where
     (==) a b = fromFVal a == fromFVal b
@@ -400,7 +404,7 @@ instance FutureVal RevocationCode where
 
 data FeatureFlag = ModificationDetection
                  | FeatureOther Int
-    deriving (Show)
+    deriving (Show, Data, Typeable)
 
 instance Eq FeatureFlag where
     (==) a b = fromFFlag a == fromFFlag b
@@ -416,28 +420,39 @@ instance FutureFlag FeatureFlag where
     toFFlag i = FeatureOther (fromIntegral i)
 
 newtype MPI = MPI {unMPI :: Integer}
-    deriving (Show, Eq)
+    deriving (Data, Eq, Show, Typeable)
 
 data SignaturePayload = SigV3 SigType Word32 EightOctetKeyId PubKeyAlgorithm HashAlgorithm Word16 [MPI]
                       | SigV4 SigType PubKeyAlgorithm HashAlgorithm [SigSubPacket] [SigSubPacket] Word16 [MPI]
                       | SigVOther Word8 ByteString
-    deriving (Show, Eq) -- FIXME
+    deriving (Data, Eq, Show, Typeable) -- FIXME
 
-data PKPayload = DeprecatedPubV3 TimeStamp V3Expiration PubKeyAlgorithm PKey
-               | PubV4 TimeStamp PubKeyAlgorithm PKey
-    deriving (Show, Eq)
+data KeyVersion = DeprecatedV3 | V4
+    deriving (Data, Eq, Ord, Show, Typeable)
+
+data PKPayload = PKPayload {
+      _keyVersion :: KeyVersion
+    , _timestamp :: TimeStamp
+    , _v3exp :: V3Expiration
+    , _pkalgo :: PubKeyAlgorithm
+    , _pubkey :: PKey
+    }
+    deriving (Data, Eq, Show, Typeable)
+
+instance Ord PKPayload where
+    compare a b = show a `compare` show b -- FIXME: this is ridiculous
 
 data SKAddendum = SUS16bit SymmetricAlgorithm S2K IV ByteString
                 | SUSSHA1 SymmetricAlgorithm S2K IV ByteString
                 | SUSym SymmetricAlgorithm IV ByteString
                 | SUUnencrypted SKey Word16
-    deriving (Show, Eq)
+    deriving (Data, Eq, Show, Typeable)
 
 data DataType = BinaryData
               | TextData
               | UTF8Data
               | OtherData Word8
-    deriving (Show)
+    deriving (Show, Data, Typeable)
 
 instance Eq DataType where
     (==) a b = fromFVal a == fromFVal b
@@ -460,18 +475,18 @@ data S2K = Simple HashAlgorithm
          | Salted HashAlgorithm Salt
          | IteratedSalted HashAlgorithm Salt Count
          | OtherS2K Word8 ByteString
-    deriving (Show, Eq) -- FIXME
+    deriving (Data, Eq, Show, Typeable) -- FIXME
 
 data UserAttrSubPacket = ImageAttribute ImageHeader ImageData
                        | OtherUASub Word8 ByteString
-    deriving (Show, Eq) -- FIXME
+    deriving (Data, Eq, Show, Typeable) -- FIXME
 
 data ImageHeader = ImageHV1 ImageFormat
-    deriving (Show, Eq)
+    deriving (Data, Eq, Show, Typeable)
 
 data ImageFormat = JPEG
                  | OtherImage Word8
-    deriving (Show)
+    deriving (Data, Show, Typeable)
 
 instance Eq ImageFormat where
     (==) a b = fromFVal a == fromFVal b
@@ -502,7 +517,7 @@ data SigType = BinarySig
              | TimestampSig
              | ThirdPartyConfirmationSig
              | OtherSig Word8
-    deriving (Show)
+    deriving (Data, Show, Typeable)
 
 instance Eq SigType where
     (==) a b = fromFVal a == fromFVal b
@@ -548,59 +563,58 @@ instance FutureVal SigType where
 data PKey = RSAPubKey RSA.PublicKey
           | DSAPubKey DSA.PublicKey
           | ElGamalPubKey [Integer]
-    deriving (Show, Eq)
+    deriving (Data, Eq, Show, Typeable)
 
 data SKey = RSAPrivateKey RSA.PrivateKey
           | DSAPrivateKey DSA.PrivateKey
           | ElGamalPrivateKey [Integer]
-    deriving (Show, Eq)
+    deriving (Data, Eq, Show, Typeable)
 
 newtype Block a = Block {unBlock :: [a]} -- so we can override cereal instance
     deriving (Show, Eq)
 
 newtype EightOctetKeyId = EightOctetKeyId {unEOKI :: ByteString}
-    deriving (Eq, Ord) -- FIXME
+    deriving (Eq, Ord, Data, Typeable) -- FIXME
 
 instance Show EightOctetKeyId where
     show = w8sToHex . B.unpack . unEOKI
 
 instance Read EightOctetKeyId where
-    readsPrec _ = map ((EightOctetKeyId . B.pack *** concat) . unzip) . splitEvery 8 . hexToW8s
+    readsPrec _ = map ((EightOctetKeyId . B.pack *** concat) . unzip) . chunksOf 8 . hexToW8s
 
 newtype TwentyOctetFingerprint = TwentyOctetFingerprint {unTOF :: ByteString}
-    deriving (Eq)
+    deriving (Eq, Ord, Data, Typeable)
 
 instance Show TwentyOctetFingerprint where
-    show = take 50 . concatMap (++" ") . concatMap (++[""]) . splitEvery 5 . splitEvery 4 . w8sToHex . B.unpack . unTOF
+    show = take 50 . concatMap (++" ") . concatMap (++[""]) . chunksOf 5 . chunksOf 4 . w8sToHex . B.unpack . unTOF
 
 instance Read TwentyOctetFingerprint where
-    readsPrec _ = map ((TwentyOctetFingerprint . B.pack *** concat) . unzip) . splitEvery 20 . hexToW8s . filter (/= ' ')
+    readsPrec _ = map ((TwentyOctetFingerprint . B.pack *** concat) . unzip) . chunksOf 20 . hexToW8s . filter (/= ' ')
 
 w8sToHex :: [Word8] -> String
 w8sToHex = map toUpper . concatMap ((\x -> if length x == 1 then '0':x else x) . flip showHex "")
 
 hexToW8s :: ReadS Word8
-hexToW8s = concatMap readHex . splitEvery 2 . map toLower
+hexToW8s = concatMap readHex . chunksOf 2 . map toLower
 
 data TK = TK {
-    tkPKP :: PKPayload
-  , tkmSKA :: Maybe SKAddendum
-  , tkRevs :: [SignaturePayload]
-  , tkUIDs :: [(String, [SignaturePayload])]
-  , tkUAts :: [([UserAttrSubPacket], [SignaturePayload])]
-  , tkSubs :: [(Pkt, SignaturePayload, Maybe SignaturePayload)]
-  }
-    deriving (Eq, Show)
+    _tkPKP :: PKPayload
+  , _tkmSKA :: Maybe SKAddendum
+  , _tkRevs :: [SignaturePayload]
+  , _tkUIDs :: [(String, [SignaturePayload])]
+  , _tkUAts :: [([UserAttrSubPacket], [SignaturePayload])]
+  , _tkSubs :: [(Pkt, SignaturePayload, Maybe SignaturePayload)]
+  } deriving (Data, Eq, Show, Typeable)
 
 instance Ord TK where
     compare a b = show a `compare` show b -- FIXME: this is ridiculous
 
-type Keyring = Map EightOctetKeyId (Set TK)
+type Keyring = IxSet TK
 
 class Packet a where
     data PacketType a :: *
     packetType :: a -> PacketType a
-    packetCode :: PacketType a -> Word8
+    packetCode :: PacketType a -> Word8
     toPkt :: a -> Pkt
     fromPkt :: Pkt -> a
 
@@ -623,14 +637,14 @@ data Pkt = PKESKPkt PacketVersion EightOctetKeyId PubKeyAlgorithm [MPI]
          | SymEncIntegrityProtectedDataPkt PacketVersion ByteString
          | ModificationDetectionCodePkt ByteString
          | OtherPacketPkt Word8 ByteString
-    deriving (Show, Eq) -- FIXME
+    deriving (Show, Eq, Data, Typeable) -- FIXME
 
 data PKESK = PKESK
-    { pkeskPacketVersion :: PacketVersion
-    , pkeskEightOctetKeyId :: EightOctetKeyId
-    , pkeskPubKeyAlgorithm :: PubKeyAlgorithm
-    , pkeskMPIs :: [MPI]
-    } deriving (Show, Eq)
+    { _pkeskPacketVersion :: PacketVersion
+    , _pkeskEightOctetKeyId :: EightOctetKeyId
+    , _pkeskPubKeyAlgorithm :: PubKeyAlgorithm
+    , _pkeskMPIs :: [MPI]
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet PKESK where
     data PacketType PKESK = PKESKType
     packetType _ = PKESKType
@@ -639,8 +653,8 @@ instance Packet PKESK where
     fromPkt (PKESKPkt a b c d) = PKESK a b c d
 
 data Signature = Signature   -- FIXME?
-    { signaturePayload :: SignaturePayload
-    } deriving (Show, Eq)
+    { _signaturePayload :: SignaturePayload
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet Signature where
     data PacketType Signature = SignatureType
     packetType _ = SignatureType
@@ -649,11 +663,11 @@ instance Packet Signature where
     fromPkt (SignaturePkt a) = Signature a
 
 data SKESK = SKESK
-    { skeskPacketVersion :: PacketVersion
-    , skeskSymmetricAlgorithm :: SymmetricAlgorithm
-    , skeskS2K :: S2K
-    , skeskMPIs :: [MPI]
-    } deriving (Show, Eq)
+    { _skeskPacketVersion :: PacketVersion
+    , _skeskSymmetricAlgorithm :: SymmetricAlgorithm
+    , _skeskS2K :: S2K
+    , _skeskMPIs :: [MPI]
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet SKESK where
     data PacketType SKESK = SKESKType
     packetType _ = SKESKType
@@ -662,13 +676,13 @@ instance Packet SKESK where
     fromPkt (SKESKPkt a b c d) = SKESK a b c d
 
 data OnePassSignature = OnePassSignature
-    { onePassSignaturePacketVersion :: PacketVersion
-    , onePassSignatureSigType :: SigType
-    , onePassSignatureHashAlgorithm :: HashAlgorithm
-    , onePassSignaturePubKeyAlgorithm :: PubKeyAlgorithm
-    , onePassSignatureEightOctetKeyId :: EightOctetKeyId
-    , onePassSignatureNestedFlag :: NestedFlag
-    } deriving (Show, Eq)
+    { _onePassSignaturePacketVersion :: PacketVersion
+    , _onePassSignatureSigType :: SigType
+    , _onePassSignatureHashAlgorithm :: HashAlgorithm
+    , _onePassSignaturePubKeyAlgorithm :: PubKeyAlgorithm
+    , _onePassSignatureEightOctetKeyId :: EightOctetKeyId
+    , _onePassSignatureNestedFlag :: NestedFlag
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet OnePassSignature where
     data PacketType OnePassSignature = OnePassSignatureType
     packetType _ = OnePassSignatureType
@@ -677,9 +691,9 @@ instance Packet OnePassSignature where
     fromPkt (OnePassSignaturePkt a b c d e f) = OnePassSignature a b c d e f
 
 data SecretKey = SecretKey
-    { secretKeyPKPayload :: PKPayload
-    , secretKeySKAddendum :: SKAddendum
-    } deriving (Show, Eq)
+    { _secretKeyPKPayload :: PKPayload
+    , _secretKeySKAddendum :: SKAddendum
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet SecretKey where
     data PacketType SecretKey = SecretKeyType
     packetType _ = SecretKeyType
@@ -688,8 +702,8 @@ instance Packet SecretKey where
     fromPkt (SecretKeyPkt a b) = SecretKey a b
 
 data PublicKey = PublicKey
-    { publicKeyPKPayload :: PKPayload
-    } deriving (Show, Eq)
+    { _publicKeyPKPayload :: PKPayload
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet PublicKey where
     data PacketType PublicKey = PublicKeyType
     packetType _ = PublicKeyType
@@ -698,9 +712,9 @@ instance Packet PublicKey where
     fromPkt (PublicKeyPkt a) = PublicKey a
 
 data SecretSubkey = SecretSubkey
-    { secretSubkeyPKPayload :: PKPayload
-    , secretSubkeySKAddendum :: SKAddendum
-    } deriving (Show, Eq)
+    { _secretSubkeyPKPayload :: PKPayload
+    , _secretSubkeySKAddendum :: SKAddendum
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet SecretSubkey where
     data PacketType SecretSubkey = SecretSubkeyType
     packetType _ = SecretSubkeyType
@@ -709,9 +723,9 @@ instance Packet SecretSubkey where
     fromPkt (SecretSubkeyPkt a b) = SecretSubkey a b
 
 data CompressedData = CompressedData
-    { compressedDataCompressionAlgorithm :: CompressionAlgorithm
-    , compressedDataPayload :: CompressedDataPayload
-    } deriving (Show, Eq)
+    { _compressedDataCompressionAlgorithm :: CompressionAlgorithm
+    , _compressedDataPayload :: CompressedDataPayload
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet CompressedData where
     data PacketType CompressedData = CompressedDataType
     packetType _ = CompressedDataType
@@ -720,8 +734,8 @@ instance Packet CompressedData where
     fromPkt (CompressedDataPkt a b) = CompressedData a b
 
 data SymEncData = SymEncData
-    { symEncDataPayload :: ByteString
-    } deriving (Show, Eq)
+    { _symEncDataPayload :: ByteString
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet SymEncData where
     data PacketType SymEncData = SymEncDataType
     packetType _ = SymEncDataType
@@ -730,8 +744,8 @@ instance Packet SymEncData where
     fromPkt (SymEncDataPkt a) = SymEncData a
 
 data Marker = Marker
-    { markerPayload :: ByteString
-    } deriving (Show, Eq)
+    { _markerPayload :: ByteString
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet Marker where
     data PacketType Marker = MarkerType
     packetType _ = MarkerType
@@ -740,11 +754,11 @@ instance Packet Marker where
     fromPkt (MarkerPkt a) = Marker a
 
 data LiteralData = LiteralData
-    { literalDataDataType :: DataType
-    , literalDataFileName :: FileName
-    , literalDataTimeStamp :: TimeStamp
-    , literalDataPayload :: ByteString
-    } deriving (Show, Eq)
+    { _literalDataDataType :: DataType
+    , _literalDataFileName :: FileName
+    , _literalDataTimeStamp :: TimeStamp
+    , _literalDataPayload :: ByteString
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet LiteralData where
     data PacketType LiteralData = LiteralDataType
     packetType _ = LiteralDataType
@@ -753,8 +767,8 @@ instance Packet LiteralData where
     fromPkt (LiteralDataPkt a b c d) = LiteralData a b c d
 
 data Trust = Trust
-    { trustPayload :: ByteString
-    } deriving (Show, Eq)
+    { _trustPayload :: ByteString
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet Trust where
     data PacketType Trust = TrustType
     packetType _ = TrustType
@@ -763,8 +777,8 @@ instance Packet Trust where
     fromPkt (TrustPkt a) = Trust a
 
 data UserId = UserId
-    { userIdPayload :: String
-    } deriving (Show, Eq)
+    { _userIdPayload :: String
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet UserId where
     data PacketType UserId = UserIdType
     packetType _ = UserIdType
@@ -773,8 +787,8 @@ instance Packet UserId where
     fromPkt (UserIdPkt a) = UserId a
 
 data PublicSubkey = PublicSubkey
-    { publicSubkeyPKPayload :: PKPayload
-    } deriving (Show, Eq)
+    { _publicSubkeyPKPayload :: PKPayload
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet PublicSubkey where
     data PacketType PublicSubkey = PublicSubkeyType
     packetType _ = PublicSubkeyType
@@ -783,8 +797,8 @@ instance Packet PublicSubkey where
     fromPkt (PublicSubkeyPkt a) = PublicSubkey a
 
 data UserAttribute = UserAttribute
-    { userAttributeSubPackets :: [UserAttrSubPacket]
-    } deriving (Show, Eq)
+    { _userAttributeSubPackets :: [UserAttrSubPacket]
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet UserAttribute where
     data PacketType UserAttribute = UserAttributeType
     packetType _ = UserAttributeType
@@ -793,9 +807,9 @@ instance Packet UserAttribute where
     fromPkt (UserAttributePkt a) = UserAttribute a
 
 data SymEncIntegrityProtectedData = SymEncIntegrityProtectedData
-    { symEncIntegrityProtectedDataPacketVersion :: PacketVersion
-    , symEncIntegrityProtectedDataPayload :: ByteString
-    } deriving (Show, Eq)
+    { _symEncIntegrityProtectedDataPacketVersion :: PacketVersion
+    , _symEncIntegrityProtectedDataPayload :: ByteString
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet SymEncIntegrityProtectedData where
     data PacketType SymEncIntegrityProtectedData = SymEncIntegrityProtectedDataType
     packetType _ = SymEncIntegrityProtectedDataType
@@ -804,8 +818,8 @@ instance Packet SymEncIntegrityProtectedData where
     fromPkt (SymEncIntegrityProtectedDataPkt a b) = SymEncIntegrityProtectedData a b
 
 data ModificationDetectionCode = ModificationDetectionCode
-    { modificationDetectionCodePayload :: ByteString
-    } deriving (Show, Eq)
+    { _modificationDetectionCodePayload :: ByteString
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet ModificationDetectionCode where
     data PacketType ModificationDetectionCode = ModificationDetectionCodeType
     packetType _ = ModificationDetectionCodeType
@@ -814,9 +828,9 @@ instance Packet ModificationDetectionCode where
     fromPkt (ModificationDetectionCodePkt a) = ModificationDetectionCode a
 
 data OtherPacket = OtherPacket
-    { otherPacketType :: Word8
-    , otherPacketPayload :: ByteString
-    } deriving (Show, Eq)
+    { _otherPacketType :: Word8
+    , _otherPacketPayload :: ByteString
+    } deriving (Data, Eq, Show, Typeable)
 instance Packet OtherPacket where
     data PacketType OtherPacket = OtherPacketType
     packetType _ = OtherPacketType
@@ -825,6 +839,28 @@ instance Packet OtherPacket where
     fromPkt (OtherPacketPkt a b) = OtherPacket a b
 
 data Verification = Verification {
-      verificationSigner :: PKPayload
-    , verificationSignature :: SignaturePayload
+      _verificationSigner :: PKPayload
+    , _verificationSignature :: SignaturePayload
     }
+
+$(makeLenses ''PKESK)
+$(makeLenses ''Signature)
+$(makeLenses ''SKESK)
+$(makeLenses ''OnePassSignature)
+$(makeLenses ''SecretKey)
+$(makeLenses ''PKPayload)
+$(makeLenses ''PublicKey)
+$(makeLenses ''SecretSubkey)
+$(makeLenses ''CompressedData)
+$(makeLenses ''SymEncData)
+$(makeLenses ''Marker)
+$(makeLenses ''LiteralData)
+$(makeLenses ''Trust)
+$(makeLenses ''UserId)
+$(makeLenses ''PublicSubkey)
+$(makeLenses ''UserAttribute)
+$(makeLenses ''SymEncIntegrityProtectedData)
+$(makeLenses ''ModificationDetectionCode)
+$(makeLenses ''OtherPacket)
+$(makeLenses ''TK)
+$(makeLenses ''Verification)
